@@ -10,6 +10,8 @@ import config
 import http
 from message import Message
 from summary import Summary
+import Logger
+
 
 class DownloadError(Exception): pass
 
@@ -136,10 +138,13 @@ class Topic(object):
         return the resulting dictionary.  If the file does not exists,
         an empty dictionary is returned.
         """
-        if os.path.exists(self._info_file_path):
-            fin = file(self._info_file_path)
-            return marshal.load(fin)
-        else:
+        try:
+            if os.path.exists(self._info_file_path):
+                return marshal.load(file(self._info_file_path))
+            else:
+                return {}
+        except EOFError, why:
+            Logger.log("EOFError in bbs.Topic._load_info() %s" % why)
             return {}
 
     def _save_info(self, info):
