@@ -25,7 +25,9 @@ def main(argv=None):
     if install.is_required():
         install.execute()
     app = Application(UserInterface)
+    Logger.log("app.start()")
     app.start()
+    Logger.log("app.start() finished")
 
 if __name__ == "__main__":
     sys.exit(main())
