@@ -1,19 +1,29 @@
-from qt import *
+# -*- coding: utf8 -*-
+
+import os
 from Queue import Queue
-import shelve
+
+from qt import *
 
 from bbs import Topic
 from renderer import Renderer
 from kittywidget import *
 from utils import *
 import config
-import os
 
 
 class MessageWidget(KittyTabWidget):
+
+    _position_file = os.path.join(config.RCDIR, "gui-messagewidget-pos")
+    _tab_menu_spec = ((u("更新"),   "requestReload"),
+                      (u("ブックマークする"), "requestBookmark"),
+                      (u("閉じる"), "requestClose"))
+
     def __init__(self, parent):
         KittyTabWidget.__init__(self, parent)
         self._font = QFont(*config.get("gui-message-font", ()))
+        self._view_pos = MarshalDict(self._position_file)
+        self._renderer = Renderer()
         # Connect
         self.connect(self,
                      PYSIGNAL("tabRightPressed"),
@@ -28,6 +38,8 @@ class MessageWidget(KittyTabWidget):
         self.connect(self._tab_menu,
                      PYSIGNAL("requestReload"),
                      self._reload_pointed_page)
+        self.connect(self._tab_menu, PYSIGNAL("requestBookmark"),
+                     self._bookmark_current_topic)
 
     def display(self, topic, messages, newtab=None):
         """Interface to display messages in topic
@@ -44,6 +56,7 @@ class MessageWidget(KittyTabWidget):
         self._font = font
         for view in self._model_view_registry.views():
             view.setFont(self._font)
+            view.setText(view.text())
         config.set("gui-message-font",
                    (unicode(font.family()), font.pointSize()))
 
@@ -61,12 +74,14 @@ class MessageWidget(KittyTabWidget):
         # new tab.
         if self._model_view_registry.has_model(topic):
             view = self._model_view_registry.view(topic)
+            self._save_position(view)
         elif newtab==False and self._model_view_registry.size()>0:
             view = self.currentPage()
             self.changeTab(view, topic.title())
+            self._save_position(view)
             self._model_view_registry.remove(self.currentPage())
         else:
-            view = QTextBrowser(self)
+            view = KittyTextBrowser(self)
             view.setFont(self._font)
             self.connect(view,
                          SIGNAL("highlighted(const QString &)"),
@@ -76,8 +91,8 @@ class MessageWidget(KittyTabWidget):
 
         self._model_view_registry.set(topic, view)
         self.showPage(view)
-        view.setText(messages)
-        view.scrollToAnchor("%d" % (topic.local_size()-50))
+        view.setText(self._renderer.render(topic, messages))
+        self._restore_position(view)
 
     def _close_pointed_page(self, page, event):
         """Close page pointed by mouse cursor"""
@@ -89,6 +104,7 @@ class MessageWidget(KittyTabWidget):
         In addition to call removePage() method, this method removes the 
         page_widget from model-view registry.
         """
+        self._save_position(page_widget)
         self.removePage(page_widget)
         self._model_view_registry.remove(page_widget)
         page_widget.close()
@@ -108,13 +124,18 @@ class MessageWidget(KittyTabWidget):
         return self._model_view_registry.model(widget)
 
     def _hover(self, href):
-        if re.match("#\d+$", str(href)):
-            index = int(str(href)[1:])
-            topic = self._current_topic()
-            html = "<qt type=detail encoding=utf8>%s</qt>" %\
-              Renderer().render(topic, index, index+1)
-            return
+        pass
 
-    def _position(self):
-        return shelve.open(os.path.join(config.RCDIR, "gui-messagewidget-pos"))
+    def _save_position(self, view):
+        topic = self._model_view_registry.model(view)
+        self._view_pos[topic.id()] = (view.contentsX(),view.contentsY())
+
+    def _restore_position(self, view):
+        topic = self._model_view_registry.model(view)
+        view.setContentsPos(0,0)
+        view.scrollBy(*self._view_pos.get(topic.id(), (0,0)))
+
+    def _bookmark_current_topic(self, view):
+        topic = self._model_view_registry.model(view)
+        self.emit(PYSIGNAL("requestBookmark"), (topic,))
 
