@@ -5,8 +5,7 @@ from bbs import Topic
 from utils import *
 
 class Renderer(object):
-    def render(self, topic, start=0, end=None):
-
+    def render(self, topic, messages, start=0, end=None):
         """Render messages in given topic and return as a single string
 
         This method iterates over every message in topic to render it as 
@@ -16,16 +15,16 @@ class Renderer(object):
         """
         assert isinstance(topic, Topic)
         if not end:
-            end = topic.local_size()
+            end = len(messages)
         return "".join(self._header(topic) +
-                       self._body(topic)[start:end] +
+                       self._body(messages)[start:end] +
                        self._footer())
 
     def _header(self, topic):
         return  ["<html><head><title>%s</title></head><body><dl>"%topic.title()]
 
-    def _body(self, topic):
-        return [self._message_division(m) for m in topic.messages()]
+    def _body(self, messages):
+        return [self._message_division(m) for m in messages]
     
     def _footer(self):
         return ["""</dl></body></html>"""]
@@ -64,5 +63,6 @@ def _ref(text):
 if __name__ == "__main__":
     # Test
     r = Renderer()
-    t = Topic("pc2.2ch.net", "tech", "1036811974")
-    print r.render(t).encode("ms932","ignore")
+    t = Topic("pc2.2ch.net", "tech", "1053955472")
+    t.update()
+    print r.render(t, [m for m in t.messages()]).encode("ms932","ignore")
