@@ -1,3 +1,5 @@
+# -*- coding: utf8 -*-
+
 from qt import *
 from Queue import Queue
 
@@ -10,9 +12,13 @@ from bbs import Board
 class TopicListWidget(KittyTabWidget):
     """Widget to display topic list"""
 
+    _tab_menu_spec = ((u("更新"),   "requestReload"),
+                      (u("閉じる"), "requestClose"))
+
     def __init__(self, parent):
         KittyTabWidget.__init__(self, parent)
         self._context_menu = self._ContextMenu(self)
+        self._newtab = True
         self.connect(self._tab_menu, PYSIGNAL("requestClose"), self._close)
         self.connect(self._tab_menu, PYSIGNAL("requestReload"),
                      self._reload_pointed_page)
@@ -25,10 +31,12 @@ class TopicListWidget(KittyTabWidget):
         self.connect(self,
                      PYSIGNAL("tabLeftDoubleClicked"),
                      self._reload_current_page)
-
-        self.connect(self._context_menu,
-                     PYSIGNAL("requestOpenMessages"),
-                     self._request_open)
+        self.connect(self._context_menu, PYSIGNAL("requestOpenInNewTab"),
+                     self._request_open_in_new_tab)
+        self.connect(self._context_menu, PYSIGNAL("requestOpenInCurrentTab"),
+                     self._request_open_in_current_tab)
+        self.connect(self._context_menu, PYSIGNAL("requestBookmark"),
+                     self._request_bookmark)
 
     def display(self, board, topics, newtab=None):
         """Interface to display the topic list of board
@@ -54,13 +62,25 @@ class TopicListWidget(KittyTabWidget):
         widget = self.currentPage()
         return self._model_view_registry.model(widget)
 
+    def _request_open_in_new_tab(self, item):
+        self._request_open(item, True)
+
+    def _request_open_in_current_tab(self, item):
+        self._request_open(item, False)
+
     def _request_open(self, item, newtab=None):
         """Request to open given item
 
         This method emits requestOpen(topic) pysignal.
         """
+        if newtab is None:
+            newtab = self._newtab
+
         self.emit(PYSIGNAL("requestOpenMessages"), (item.topic(), newtab))
 
+    def _request_bookmark(self, item):
+        self.emit(PYSIGNAL("requestAppendBookmark"), (item.topic(),))
+
     def _display(self, board, topics, newtab):
         """Display the topic list of board, used from sub thread.
         
@@ -155,20 +175,24 @@ class TopicListWidget(KittyTabWidget):
             return self._topic
 
     class _ContextMenu(QPopupMenu):
-        def __init__(self, parent):
-            QPopupMenu.__init__(self, parent)
-            self._view = parent
-            self._target_item = None
-            self.insertItem(u("新しいタブで開く"), self._open_in_new_tab)
-            self.insertItem(u("今のタブで開く"), self._open_in_current_tab)
+        def __init__(self, *args):
+            QPopupMenu.__init__(self, *args)
+            self._signal = {}
+            for text, signal in [(u("新しいタブで開く"),
+                                  "requestOpenInNewTab"),
+                                 (u("今のタブで開く"),
+                                  "requestOpenInCurrentTab"),
+                                 (u("ブックマークする"),
+                                  "requestBookmark")]:
+                item_id = self.insertItem(text)
+                self._signal[item_id] = signal
+            self._item = None
+            self.connect(self, SIGNAL("activated(int)"), self._activated)
+
+        def _activated(self, item_id):
+            self.emit(PYSIGNAL(self._signal[item_id]), (self._item,))
 
         def popup(self, item, position):
-            self._target_item = item
+            self._item = item
             QPopupMenu.popup(self, position)
-
-        def _open_in_new_tab(self):
-            self.emit(PYSIGNAL("requestOpenMessages"),(self._target_item,True))
-
-        def _open_in_current_tab(self):
-            self.emit(PYSIGNAL("requestOpenMessages"),(self._target_item,False))
-
+        
